#include "TMHistogrammingNumberClustersInner.h"
#include "HenrikClusterFinderSPD.h"
#include <TStyle.h>
#include <TPad.h>
#include <TH1F.h>

//_____________________________________________________________________
TMHistogrammingNumberClustersInner::TMHistogrammingNumberClustersInner() :
  TMHistogramming(), fRangeX(25)
{ 

  fTheHisto = NULL;
  CreateHistos();

}
//_____________________________________________________________________
TMHistogrammingNumberClustersInner::~TMHistogrammingNumberClustersInner() {
  if (fTheHisto!=NULL) delete fTheHisto;
}
//_____________________________________________________________________
void TMHistogrammingNumberClustersInner::CreateHistos() {

  fTheHisto = new TH1F("NumberClustersInnertot","Cluster Multiplicity Inner Layer",
		       200,-0.5,fRangeX-0.5);
  fTheHisto->SetXTitle("Number of Clusters");
  fTheHisto->SetStats(0);
  fTheHisto->SetFillColor(kYellow);
  fTheHisto->SetBit(TH1::kCanRebin);

}
//_____________________________________________________________________
void TMHistogrammingNumberClustersInner::ProcessClusters(HenrikClusterFinderSPD* clFinder) {
  if (fEventAccept) {
    UInt_t clusterCount = 0;
    for (UInt_t mod=0; mod<80; mod++) {
      clusterCount+= clFinder->GetClusterCount(mod);
    }
    
  
    if (clusterCount >= fRangeX) {
      while (clusterCount >= fRangeX) {
	fRangeX*=2;
      }
      RebinHistos();
    }
    
    fTheHisto->Fill(clusterCount);
  }
}
//_____________________________________________________________________
void TMHistogrammingNumberClustersInner::Draw(TPad *pad) {
  pad->cd();
  pad->SetLeftMargin(0.15);
  pad->SetRightMargin(0.15);
  pad->SetBorderSize(0);
  pad->SetBorderMode(0);

  fTheHisto->Draw();

  fNeedRedraw = kFALSE;
}
//_____________________________________________________________________
void TMHistogrammingNumberClustersInner::Reset() {
  fRangeX=25;

  if (fTheHisto!=NULL) delete fTheHisto;
  fTheHisto = NULL;

  CreateHistos();

  fNeedRedraw = kTRUE;

  TMHistogramming::Reset();
}
//_____________________________________________________________________
void TMHistogrammingNumberClustersInner::RebinHistos() {
  fTheHisto->RebinAxis((Double_t)fRangeX,fTheHisto->GetXaxis());
}



