#ifndef TMSetupDisplayPixel_H
#define TMSetupDisplayPixel_H

#include "SPDMood.h"
#include <TGFrame.h>
#include <TGButton.h>
#include <TGButtonGroup.h>
#include <TGTab.h>
#include <TGLabel.h>

const UInt_t gNrHistogramming = 1385; // ADD CODE HERE   (if incorrect, warning when starting SPDmood)

class TMHistogramming;

class TMSetupDisplays : public TGMainFrame {
    
 public:
    
  TMSetupDisplays(const TGWindow *main, UInt_t wid, UInt_t hei);
  virtual ~TMSetupDisplays();

  static  void*  THROpenDisplay1(void *arg);
  static  void*  THROpenDisplay4(void *arg);
  static  void*  THRCloseDisplay2(void *arg);
  static  void*  THRCloseDisplay3(void *arg);
  static  void*  THRCloneDisplay3(void *arg);


  virtual void   SaveDefaultSetup();
  virtual void   SaveSetupChoice();
  virtual void   SaveSetup(const Char_t *fileName);
  virtual void   LoadDefaultSetup();
  virtual void   LoadSetupChoice();
  virtual void   LoadSetup(const Char_t *fileName);
  virtual void   ClearSetup();

  virtual Bool_t ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2);

  virtual void   ProcessPlusHistogramming(UInt_t id);
  virtual void   ProcessPlusHistogramming(UInt_t id, UInt_t span);
  virtual void   ProcessMinusHistogramming(UInt_t id);
  virtual void   ProcessMinusHistogramming(UInt_t id, UInt_t span);
  virtual void   ProcessOpenDisplayID(UInt_t id, UInt_t span=1);
  virtual void   ProcessOpenDisplay(UInt_t d_id);
  virtual void   ProcessOpenDisplay(UInt_t d_id, UInt_t plotNr, UInt_t cloneId=0);
  virtual void   ProcessOpenDisplay(UInt_t d_id, UInt_t plotNr, UInt_t cloneId, TMHistogramming *hist);
  virtual void   ProcessCloseDisplay(UInt_t d_id, UInt_t plotNr);
  virtual void   ProcessCloseDisplay(UInt_t d_id, UInt_t plotNr, UInt_t cloneId);
  virtual void   ProcessCloseAllDisplays(Bool_t hold=kTRUE);
  virtual void   ProcessKillDisplay(UInt_t d_id, UInt_t plotNr);
  virtual void   ProcessAutoResetDisplay(UInt_t d_id, UInt_t plotNr, Bool_t autoreset);
  virtual void   ProcessPauseDisplay(UInt_t d_id, UInt_t plotNr, Bool_t pause);
  virtual void   ProcessCloneDisplay(UInt_t d_id, UInt_t plotNr, TMHistogramming *hist);


  virtual void   ThreadProcessOpenDisplay(UInt_t d_id);
  virtual void   ThreadProcessOpenDisplay(UInt_t d_id, UInt_t plotNr, UInt_t cloneId, TMHistogramming *hist);
  virtual void   ThreadProcessOpenDisplayID(UInt_t id, UInt_t span=1);
  virtual void   ThreadProcessCloseDisplay(UInt_t d_id, UInt_t plotNr);
  virtual void   ThreadProcessCloseDisplay(UInt_t d_id, UInt_t plotNr, UInt_t cloneId);
  virtual void   ThreadProcessCloneDisplay(UInt_t d_id, UInt_t plotNr, TMHistogramming *hist);

  virtual void   ProcessDeadNoisy();

  virtual void   UpdateNrLabel(UInt_t id);
  virtual void   UpdateNrLabelLater(UInt_t id, UInt_t span=1);

  virtual UInt_t GetNewCloneId();
  static  UInt_t GetChipID(UInt_t eq, UInt_t hs, UInt_t chip);

  virtual UInt_t GetThreadInt1() {return fThreadInt1;}
  virtual void   SetThreadInt1(UInt_t val) {fThreadInt1=val;}
  virtual UInt_t GetThreadInt2() {return fThreadInt2;}
  virtual void   SetThreadInt2(UInt_t val) {fThreadInt2=val;}
  virtual UInt_t GetThreadInt3() {return fThreadInt3;}
  virtual void   SetThreadInt3(UInt_t val) {fThreadInt3=val;}
  virtual Bool_t GetThreadBool1() {return fThreadBool1;}
  virtual void   SetThreadBool1(UInt_t val) {fThreadBool1=val;}
  virtual TMHistogramming* GetThreadHist1() {return fThreadHist1;}
  virtual void   SetThreadHist1(TMHistogramming *val) {fThreadHist1=val;}

  private:

  UInt_t fNrHistogramming[gNrHistogramming];
  UInt_t fCloneCounter;

  // thread temp vars:
  UInt_t fThreadInt1;
  UInt_t fThreadInt2;
  UInt_t fThreadInt3;
  Bool_t fThreadBool1;
  TMHistogramming *fThreadHist1;

  Bool_t fPleaseWait;

  Pixel_t white, red, black;

  TGTab            *fMainTab;

  // pixel tab
  TGCompositeFrame *fPixelTab;
  TGCompositeFrame *fPixelFrame;

  // global tab
  TGCompositeFrame *fGlobalTab;
  TGCompositeFrame *fGlobalFrame;

  // fastor tab
  TGCompositeFrame *fFastorTab;
  TGCompositeFrame *fFastorFrame;

  // cons error tab
  TGCompositeFrame *fCErrorTab;
  TGCompositeFrame *fCErrorFrame;

  // dump tab
  TGCompositeFrame *fDumpTab;
  TGCompositeFrame *fDumpFrame;

  // new disp tab
  TGCompositeFrame *fNewTab;
  TGCompositeFrame *fNewFrame;


  // hitmaps
  TGCompositeFrame *fHitMap;
  TGLabel          *fHitMapTitle;
  TGTextButton     *fHitMapButtonAllPlus, *fHitMapButtonAllMinus, *fHitMapButtonAllShow;

  // hits per event
  TGCompositeFrame *fHitsPerEvent;
  TGLabel          *fHitsPerEventTitle;
  TGTextButton     *fHitsPerEventButtonAllPlus, *fHitsPerEventButtonAllMinus, *fHitsPerEventButtonAllShow;

  // events with hits
  TGCompositeFrame *fEventsWithHits;
  TGLabel          *fEventsWithHitsTitle;
  TGTextButton     *fEventsWithHitsButtonAllPlus, *fEventsWithHitsButtonAllMinus, *fEventsWithHitsButtonAllShow;

  // global clu
  TGCompositeFrame *fGlobalClu;
  TGLabel          *fGlobalCluTitle;
  TGTextButton     *fGlobalCluButtonAllPlus, *fGlobalCluButtonAllMinus, *fGlobalCluButtonAllShow;

  // global pix
  TGCompositeFrame *fGlobalPix;
  TGLabel          *fGlobalPixTitle;
  TGTextButton     *fGlobalPixButtonAllPlus, *fGlobalPixButtonAllMinus, *fGlobalPixButtonAllShow;

  // global other
  TGCompositeFrame *fGlobalOther;
  TGLabel          *fGlobalOtherTitle;
  TGTextButton     *fGlobalOtherButtonAllPlus, *fGlobalOtherButtonAllMinus, *fGlobalOtherButtonAllShow;

  // fastor eq
  TGCompositeFrame *fFastorEq;
  TGLabel          *fFastorEqTitle;
  TGTextButton     *fFastorEqButtonAllPlus, *fFastorEqButtonAllMinus, *fFastorEqButtonAllShow;

  // fastor problem
  TGCompositeFrame *fFastorProblem;
  TGLabel          *fFastorProblemTitle;
  TGTextButton     *fFastorProblemButtonAllPlus, *fFastorProblemButtonAllMinus, *fFastorProblemButtonAllShow;

  // fastor additional
  TGCompositeFrame *fFastorAdditional;
  TGLabel          *fFastorAdditionalTitle;
  TGTextButton     *fFastorAdditionalButtonAllPlus, *fFastorAdditionalButtonAllMinus, *fFastorAdditionalButtonAllShow;

  // cons error
  TGCompositeFrame *fCErrorOne;
  TGLabel          *fCErrorOneTitle, *fCErrorOneTitle2;
  TGTextButton     *fCErrorOneButtonAllPlus, *fCErrorOneButtonAllMinus, *fCErrorOneButtonAllShow;

  TGCompositeFrame *fCErrorTwo;
  TGLabel          *fCErrorTwoTitle;
  TGTextButton     *fCErrorTwoButtonAllPlus, *fCErrorTwoButtonAllMinus, *fCErrorTwoButtonAllShow;

  TGCompositeFrame *fCErrorThree;
  TGLabel          *fCErrorThreeTitle;
  TGTextButton     *fCErrorThreeButtonAllPlus, *fCErrorThreeButtonAllMinus, *fCErrorThreeButtonAllShow;

  // dump main
  TGCompositeFrame *fDumpMain;
  TGLabel          *fDumpMainTitle;
  TGTextButton     *fDumpMainButtonAllPlus, *fDumpMainButtonAllMinus, *fDumpMainButtonAllShow;



  // new stuff
  TGCompositeFrame *fNewStuff;
  TGLabel          *fNewStuffTitle;
  //  TGTextButton     *fNewStuffButtonAllPlus, *fNewStuffButtonAllMinus, *fNewStuffButtonAllShow;




  // ADD CODE HERE maybe...






  TGLabel          *fLabelEntry[gNrHistogramming];
  TGTextButton     *fPlusEntry[gNrHistogramming];
  TGTextButton     *fMinusEntry[gNrHistogramming];
  TGLabel          *fNrPlotsEntry[gNrHistogramming];
  TGTextButton     *fShowEntry[gNrHistogramming];



  TGLayoutHints	   *fLayout, *fLayout2, *fLayout3, *fLayout4, *fLayout5;
  TGLayoutHints	   *fLayout6, *fLayout7 ;


  virtual void AddEntry(UInt_t id, TGCompositeFrame *parent, const Char_t *label);

  enum menus_and_buttons {
    HITMAP_ALLPLUS, HITMAP_ALLMINUS, HITMAP_ALLSHOW, 
    HITSPEREVENT_ALLPLUS, HITSPEREVENT_ALLMINUS, HITSPEREVENT_ALLSHOW,
    EVENTSWITHHITS_ALLPLUS, EVENTSWITHHITS_ALLMINUS, EVENTSWITHHITS_ALLSHOW,
    GLOBAL_CLU_ALLPLUS, GLOBAL_CLU_ALLMINUS, GLOBAL_CLU_ALLSHOW, 
    GLOBAL_PIX_ALLPLUS, GLOBAL_PIX_ALLMINUS, GLOBAL_PIX_ALLSHOW, 
    GLOBAL_OTHER_ALLPLUS, GLOBAL_OTHER_ALLMINUS, GLOBAL_OTHER_ALLSHOW, 
    FASTOR_EQ_ALLPLUS, FASTOR_EQ_ALLMINUS, FASTOR_EQ_ALLSHOW,
    FASTOR_PROBLEM_EQ_ALLPLUS, FASTOR_PROBLEM_EQ_ALLMINUS, FASTOR_PROBLEM_EQ_ALLSHOW,
    FASTOR_ADDITIONAL_ALLPLUS, FASTOR_ADDITIONAL_ALLMINUS, FASTOR_ADDITIONAL_ALLSHOW,
    CERROR_ONE_ALLPLUS, CERROR_ONE_ALLMINUS, CERROR_ONE_ALLSHOW,
    CERROR_TWO_ALLPLUS, CERROR_TWO_ALLMINUS, CERROR_TWO_ALLSHOW,
    CERROR_THREE_ALLPLUS, CERROR_THREE_ALLMINUS, CERROR_THREE_ALLSHOW,
    DUMP_MAIN_ALLPLUS, DUMP_MAIN_ALLMINUS, DUMP_MAIN_ALLSHOW,
    FILE_LOAD_DEFAULT_SETUP, FILE_SAVE_DEFAULT_SETUP, FILE_LOAD_SETUP, FILE_SAVE_SETUP, FILE_CLEAR_SETUP

    // ADD CODE HERE maybe...
  };

  enum defs {
    // should be enough for unique id:
    ZERO_TO_DISP_ID_SHIFT  = 100000,
    ZERO_TO_PLUS_ID_SHIFT  = 200000,
    ZERO_TO_MINUS_ID_SHIFT = 300000
  };

  enum ids {

    // global cluster
    GLOBAL_CLUSTER_XY,
    GLOBAL_CLUSTER_ZY,
    GLOBAL_CLUSTER_XYZ,
    CLUSTER_MULTIPLICITY,
    CLUSTER_INNER_VS_OUTER,
    CLUSTERS_PER_EQ_INNER,
    CLUSTERS_PER_EQ_OUTER,
    CLUSTERS_PER_HS_A,
    CLUSTERS_PER_HS_C,
    CLUSTERS_PER_STAVE,
    CLUSTER_SIZE_INNER,
    CLUSTER_SIZE_OUTER,
    CLUSTER_TYPE_INNER,
    CLUSTER_TYPE_OUTER,
    CLUSTER_WIDTH_Z,
    CLUSTER_WIDTH_PHI,
    CLUSTER_WIDTH_ZVSPHI_INNER,
    CLUSTER_WIDTH_ZVSPHI_OUTER,

    // global pixel
    GLOBAL_PIXEL_XY,
    GLOBAL_PIXEL_ZY,
    GLOBAL_PIXEL_XYZ,
    PIXEL_MULTIPLICITY,
    PIXEL_MULTIPLICITYHS,
    PIXEL_MULTIPLICITYCHIP,
    GLOBAL_PIXEL_PHIDIST,
    GLOBAL_PIXEL_ZDIST ,
    GLOBAL_PIXEL_PHIDISTREF,
    GLOBAL_PIXEL_ZDISTREF,

    // global other
    BC_MOD_4,

    // hitmaps
    HITMAPEQ_0, HITMAPEQ_1, HITMAPEQ_2, HITMAPEQ_3, HITMAPEQ_4, 
    HITMAPEQ_5, HITMAPEQ_6, HITMAPEQ_7, HITMAPEQ_8, HITMAPEQ_9, 
    HITMAPEQ_10, HITMAPEQ_11, HITMAPEQ_12, HITMAPEQ_13, HITMAPEQ_14, 
    HITMAPEQ_15, HITMAPEQ_16, HITMAPEQ_17, HITMAPEQ_18, HITMAPEQ_19, 

    // hits per event
    HITSPEREVENTALL,
    HITSPEREVENTEQ_0, HITSPEREVENTEQ_1, HITSPEREVENTEQ_2, HITSPEREVENTEQ_3, HITSPEREVENTEQ_4, 
    HITSPEREVENTEQ_5, HITSPEREVENTEQ_6, HITSPEREVENTEQ_7, HITSPEREVENTEQ_8, HITSPEREVENTEQ_9, 
    HITSPEREVENTEQ_10, HITSPEREVENTEQ_11, HITSPEREVENTEQ_12, HITSPEREVENTEQ_13, HITSPEREVENTEQ_14, 
    HITSPEREVENTEQ_15, HITSPEREVENTEQ_16, HITSPEREVENTEQ_17, HITSPEREVENTEQ_18, HITSPEREVENTEQ_19, 

    // events with hits
    EVENTSWITHHITSALL,
    EVENTSWITHHITSEQ_0, EVENTSWITHHITSEQ_1, EVENTSWITHHITSEQ_2, EVENTSWITHHITSEQ_3, EVENTSWITHHITSEQ_4, 
    EVENTSWITHHITSEQ_5, EVENTSWITHHITSEQ_6, EVENTSWITHHITSEQ_7, EVENTSWITHHITSEQ_8, EVENTSWITHHITSEQ_9, 
    EVENTSWITHHITSEQ_10, EVENTSWITHHITSEQ_11, EVENTSWITHHITSEQ_12, EVENTSWITHHITSEQ_13, EVENTSWITHHITSEQ_14, 
    EVENTSWITHHITSEQ_15, EVENTSWITHHITSEQ_16, EVENTSWITHHITSEQ_17, EVENTSWITHHITSEQ_18, EVENTSWITHHITSEQ_19, 

    // fastor
    FASTOR_ALL,
    FASTOR_EQ_0, FASTOR_EQ_1, FASTOR_EQ_2, FASTOR_EQ_3, FASTOR_EQ_4, 
    FASTOR_EQ_5, FASTOR_EQ_6, FASTOR_EQ_7, FASTOR_EQ_8, FASTOR_EQ_9, 
    FASTOR_EQ_10, FASTOR_EQ_11, FASTOR_EQ_12, FASTOR_EQ_13, FASTOR_EQ_14, 
    FASTOR_EQ_15, FASTOR_EQ_16, FASTOR_EQ_17, FASTOR_EQ_18, FASTOR_EQ_19, 
    FASTOR_PROBLEM_ALL,
    FASTOR_PROBLEM_EQ_0, FASTOR_PROBLEM_EQ_1, FASTOR_PROBLEM_EQ_2, FASTOR_PROBLEM_EQ_3, FASTOR_PROBLEM_EQ_4, 
    FASTOR_PROBLEM_EQ_5, FASTOR_PROBLEM_EQ_6, FASTOR_PROBLEM_EQ_7, FASTOR_PROBLEM_EQ_8, FASTOR_PROBLEM_EQ_9, 
    FASTOR_PROBLEM_EQ_10, FASTOR_PROBLEM_EQ_11, FASTOR_PROBLEM_EQ_12, FASTOR_PROBLEM_EQ_13, FASTOR_PROBLEM_EQ_14, 
    FASTOR_PROBLEM_EQ_15, FASTOR_PROBLEM_EQ_16, FASTOR_PROBLEM_EQ_17, FASTOR_PROBLEM_EQ_18, FASTOR_PROBLEM_EQ_19, 
    FASTOR_INNER_VS_OUTER,

    // consistency checks
    CERROR_VS_EQ,
    CERROR_VS_EC,
    CERROR_FRACTION_VS_EQ,
    CERROR_FRACTION_VS_EC,
    CERROR_VS_EVENT,
    CERROR_POS,
    CERROR_MESSAGES,

    CERROR_VS_EC_EQ_0, CERROR_VS_EC_EQ_1, CERROR_VS_EC_EQ_2, CERROR_VS_EC_EQ_3, CERROR_VS_EC_EQ_4, 
    CERROR_VS_EC_EQ_5, CERROR_VS_EC_EQ_6, CERROR_VS_EC_EQ_7, CERROR_VS_EC_EQ_8, CERROR_VS_EC_EQ_9, 
    CERROR_VS_EC_EQ_10, CERROR_VS_EC_EQ_11, CERROR_VS_EC_EQ_12, CERROR_VS_EC_EQ_13, CERROR_VS_EC_EQ_14, 
    CERROR_VS_EC_EQ_15, CERROR_VS_EC_EQ_16, CERROR_VS_EC_EQ_17, CERROR_VS_EC_EQ_18, CERROR_VS_EC_EQ_19, 

    CERROR_FRACTION_VS_EC_EQ_0, CERROR_FRACTION_VS_EC_EQ_1, CERROR_FRACTION_VS_EC_EQ_2, CERROR_FRACTION_VS_EC_EQ_3, 
    CERROR_FRACTION_VS_EC_EQ_4, CERROR_FRACTION_VS_EC_EQ_5, CERROR_FRACTION_VS_EC_EQ_6, CERROR_FRACTION_VS_EC_EQ_7, 
    CERROR_FRACTION_VS_EC_EQ_8, CERROR_FRACTION_VS_EC_EQ_9, CERROR_FRACTION_VS_EC_EQ_10, CERROR_FRACTION_VS_EC_EQ_11, 
    CERROR_FRACTION_VS_EC_EQ_12, CERROR_FRACTION_VS_EC_EQ_13, CERROR_FRACTION_VS_EC_EQ_14, CERROR_FRACTION_VS_EC_EQ_15, 
    CERROR_FRACTION_VS_EC_EQ_16, CERROR_FRACTION_VS_EC_EQ_17, CERROR_FRACTION_VS_EC_EQ_18, CERROR_FRACTION_VS_EC_EQ_19, 


    // dump
    DUMP_PAYLOAD, DUMP_HEADER_SHORT, DUMP_HEADER_LONG, DUMP_CALIB_HEADER, 


    // new stuff



    // ADD CODE HERE



    HITMAPCHIP_0_0_0, HITMAPCHIP_0_0_1, HITMAPCHIP_0_0_2, HITMAPCHIP_0_0_3, HITMAPCHIP_0_0_4,
    HITMAPCHIP_0_0_5, HITMAPCHIP_0_0_6, HITMAPCHIP_0_0_7, HITMAPCHIP_0_0_8, HITMAPCHIP_0_0_9,
    HITMAPCHIP_0_1_0, HITMAPCHIP_0_1_1, HITMAPCHIP_0_1_2, HITMAPCHIP_0_1_3, HITMAPCHIP_0_1_4,
    HITMAPCHIP_0_1_5, HITMAPCHIP_0_1_6, HITMAPCHIP_0_1_7, HITMAPCHIP_0_1_8, HITMAPCHIP_0_1_9,
    HITMAPCHIP_0_2_0, HITMAPCHIP_0_2_1, HITMAPCHIP_0_2_2, HITMAPCHIP_0_2_3, HITMAPCHIP_0_2_4,
    HITMAPCHIP_0_2_5, HITMAPCHIP_0_2_6, HITMAPCHIP_0_2_7, HITMAPCHIP_0_2_8, HITMAPCHIP_0_2_9,
    HITMAPCHIP_0_3_0, HITMAPCHIP_0_3_1, HITMAPCHIP_0_3_2, HITMAPCHIP_0_3_3, HITMAPCHIP_0_3_4,
    HITMAPCHIP_0_3_5, HITMAPCHIP_0_3_6, HITMAPCHIP_0_3_7, HITMAPCHIP_0_3_8, HITMAPCHIP_0_3_9,
    HITMAPCHIP_0_4_0, HITMAPCHIP_0_4_1, HITMAPCHIP_0_4_2, HITMAPCHIP_0_4_3, HITMAPCHIP_0_4_4,
    HITMAPCHIP_0_4_5, HITMAPCHIP_0_4_6, HITMAPCHIP_0_4_7, HITMAPCHIP_0_4_8, HITMAPCHIP_0_4_9,
    HITMAPCHIP_0_5_0, HITMAPCHIP_0_5_1, HITMAPCHIP_0_5_2, HITMAPCHIP_0_5_3, HITMAPCHIP_0_5_4,
    HITMAPCHIP_0_5_5, HITMAPCHIP_0_5_6, HITMAPCHIP_0_5_7, HITMAPCHIP_0_5_8, HITMAPCHIP_0_5_9,
    HITMAPCHIP_1_0_0, HITMAPCHIP_1_0_1, HITMAPCHIP_1_0_2, HITMAPCHIP_1_0_3, HITMAPCHIP_1_0_4,
    HITMAPCHIP_1_0_5, HITMAPCHIP_1_0_6, HITMAPCHIP_1_0_7, HITMAPCHIP_1_0_8, HITMAPCHIP_1_0_9,
    HITMAPCHIP_1_1_0, HITMAPCHIP_1_1_1, HITMAPCHIP_1_1_2, HITMAPCHIP_1_1_3, HITMAPCHIP_1_1_4,
    HITMAPCHIP_1_1_5, HITMAPCHIP_1_1_6, HITMAPCHIP_1_1_7, HITMAPCHIP_1_1_8, HITMAPCHIP_1_1_9,
    HITMAPCHIP_1_2_0, HITMAPCHIP_1_2_1, HITMAPCHIP_1_2_2, HITMAPCHIP_1_2_3, HITMAPCHIP_1_2_4,
    HITMAPCHIP_1_2_5, HITMAPCHIP_1_2_6, HITMAPCHIP_1_2_7, HITMAPCHIP_1_2_8, HITMAPCHIP_1_2_9,
    HITMAPCHIP_1_3_0, HITMAPCHIP_1_3_1, HITMAPCHIP_1_3_2, HITMAPCHIP_1_3_3, HITMAPCHIP_1_3_4,
    HITMAPCHIP_1_3_5, HITMAPCHIP_1_3_6, HITMAPCHIP_1_3_7, HITMAPCHIP_1_3_8, HITMAPCHIP_1_3_9,
    HITMAPCHIP_1_4_0, HITMAPCHIP_1_4_1, HITMAPCHIP_1_4_2, HITMAPCHIP_1_4_3, HITMAPCHIP_1_4_4,
    HITMAPCHIP_1_4_5, HITMAPCHIP_1_4_6, HITMAPCHIP_1_4_7, HITMAPCHIP_1_4_8, HITMAPCHIP_1_4_9,
    HITMAPCHIP_1_5_0, HITMAPCHIP_1_5_1, HITMAPCHIP_1_5_2, HITMAPCHIP_1_5_3, HITMAPCHIP_1_5_4,
    HITMAPCHIP_1_5_5, HITMAPCHIP_1_5_6, HITMAPCHIP_1_5_7, HITMAPCHIP_1_5_8, HITMAPCHIP_1_5_9,
    HITMAPCHIP_2_0_0, HITMAPCHIP_2_0_1, HITMAPCHIP_2_0_2, HITMAPCHIP_2_0_3, HITMAPCHIP_2_0_4,
    HITMAPCHIP_2_0_5, HITMAPCHIP_2_0_6, HITMAPCHIP_2_0_7, HITMAPCHIP_2_0_8, HITMAPCHIP_2_0_9,
    HITMAPCHIP_2_1_0, HITMAPCHIP_2_1_1, HITMAPCHIP_2_1_2, HITMAPCHIP_2_1_3, HITMAPCHIP_2_1_4,
    HITMAPCHIP_2_1_5, HITMAPCHIP_2_1_6, HITMAPCHIP_2_1_7, HITMAPCHIP_2_1_8, HITMAPCHIP_2_1_9,
    HITMAPCHIP_2_2_0, HITMAPCHIP_2_2_1, HITMAPCHIP_2_2_2, HITMAPCHIP_2_2_3, HITMAPCHIP_2_2_4,
    HITMAPCHIP_2_2_5, HITMAPCHIP_2_2_6, HITMAPCHIP_2_2_7, HITMAPCHIP_2_2_8, HITMAPCHIP_2_2_9,
    HITMAPCHIP_2_3_0, HITMAPCHIP_2_3_1, HITMAPCHIP_2_3_2, HITMAPCHIP_2_3_3, HITMAPCHIP_2_3_4,
    HITMAPCHIP_2_3_5, HITMAPCHIP_2_3_6, HITMAPCHIP_2_3_7, HITMAPCHIP_2_3_8, HITMAPCHIP_2_3_9,
    HITMAPCHIP_2_4_0, HITMAPCHIP_2_4_1, HITMAPCHIP_2_4_2, HITMAPCHIP_2_4_3, HITMAPCHIP_2_4_4,
    HITMAPCHIP_2_4_5, HITMAPCHIP_2_4_6, HITMAPCHIP_2_4_7, HITMAPCHIP_2_4_8, HITMAPCHIP_2_4_9,
    HITMAPCHIP_2_5_0, HITMAPCHIP_2_5_1, HITMAPCHIP_2_5_2, HITMAPCHIP_2_5_3, HITMAPCHIP_2_5_4,
    HITMAPCHIP_2_5_5, HITMAPCHIP_2_5_6, HITMAPCHIP_2_5_7, HITMAPCHIP_2_5_8, HITMAPCHIP_2_5_9,
    HITMAPCHIP_3_0_0, HITMAPCHIP_3_0_1, HITMAPCHIP_3_0_2, HITMAPCHIP_3_0_3, HITMAPCHIP_3_0_4,
    HITMAPCHIP_3_0_5, HITMAPCHIP_3_0_6, HITMAPCHIP_3_0_7, HITMAPCHIP_3_0_8, HITMAPCHIP_3_0_9,
    HITMAPCHIP_3_1_0, HITMAPCHIP_3_1_1, HITMAPCHIP_3_1_2, HITMAPCHIP_3_1_3, HITMAPCHIP_3_1_4,
    HITMAPCHIP_3_1_5, HITMAPCHIP_3_1_6, HITMAPCHIP_3_1_7, HITMAPCHIP_3_1_8, HITMAPCHIP_3_1_9,
    HITMAPCHIP_3_2_0, HITMAPCHIP_3_2_1, HITMAPCHIP_3_2_2, HITMAPCHIP_3_2_3, HITMAPCHIP_3_2_4,
    HITMAPCHIP_3_2_5, HITMAPCHIP_3_2_6, HITMAPCHIP_3_2_7, HITMAPCHIP_3_2_8, HITMAPCHIP_3_2_9,
    HITMAPCHIP_3_3_0, HITMAPCHIP_3_3_1, HITMAPCHIP_3_3_2, HITMAPCHIP_3_3_3, HITMAPCHIP_3_3_4,
    HITMAPCHIP_3_3_5, HITMAPCHIP_3_3_6, HITMAPCHIP_3_3_7, HITMAPCHIP_3_3_8, HITMAPCHIP_3_3_9,
    HITMAPCHIP_3_4_0, HITMAPCHIP_3_4_1, HITMAPCHIP_3_4_2, HITMAPCHIP_3_4_3, HITMAPCHIP_3_4_4,
    HITMAPCHIP_3_4_5, HITMAPCHIP_3_4_6, HITMAPCHIP_3_4_7, HITMAPCHIP_3_4_8, HITMAPCHIP_3_4_9,
    HITMAPCHIP_3_5_0, HITMAPCHIP_3_5_1, HITMAPCHIP_3_5_2, HITMAPCHIP_3_5_3, HITMAPCHIP_3_5_4,
    HITMAPCHIP_3_5_5, HITMAPCHIP_3_5_6, HITMAPCHIP_3_5_7, HITMAPCHIP_3_5_8, HITMAPCHIP_3_5_9,
    HITMAPCHIP_4_0_0, HITMAPCHIP_4_0_1, HITMAPCHIP_4_0_2, HITMAPCHIP_4_0_3, HITMAPCHIP_4_0_4,
    HITMAPCHIP_4_0_5, HITMAPCHIP_4_0_6, HITMAPCHIP_4_0_7, HITMAPCHIP_4_0_8, HITMAPCHIP_4_0_9,
    HITMAPCHIP_4_1_0, HITMAPCHIP_4_1_1, HITMAPCHIP_4_1_2, HITMAPCHIP_4_1_3, HITMAPCHIP_4_1_4,
    HITMAPCHIP_4_1_5, HITMAPCHIP_4_1_6, HITMAPCHIP_4_1_7, HITMAPCHIP_4_1_8, HITMAPCHIP_4_1_9,
    HITMAPCHIP_4_2_0, HITMAPCHIP_4_2_1, HITMAPCHIP_4_2_2, HITMAPCHIP_4_2_3, HITMAPCHIP_4_2_4,
    HITMAPCHIP_4_2_5, HITMAPCHIP_4_2_6, HITMAPCHIP_4_2_7, HITMAPCHIP_4_2_8, HITMAPCHIP_4_2_9,
    HITMAPCHIP_4_3_0, HITMAPCHIP_4_3_1, HITMAPCHIP_4_3_2, HITMAPCHIP_4_3_3, HITMAPCHIP_4_3_4,
    HITMAPCHIP_4_3_5, HITMAPCHIP_4_3_6, HITMAPCHIP_4_3_7, HITMAPCHIP_4_3_8, HITMAPCHIP_4_3_9,
    HITMAPCHIP_4_4_0, HITMAPCHIP_4_4_1, HITMAPCHIP_4_4_2, HITMAPCHIP_4_4_3, HITMAPCHIP_4_4_4,
    HITMAPCHIP_4_4_5, HITMAPCHIP_4_4_6, HITMAPCHIP_4_4_7, HITMAPCHIP_4_4_8, HITMAPCHIP_4_4_9,
    HITMAPCHIP_4_5_0, HITMAPCHIP_4_5_1, HITMAPCHIP_4_5_2, HITMAPCHIP_4_5_3, HITMAPCHIP_4_5_4,
    HITMAPCHIP_4_5_5, HITMAPCHIP_4_5_6, HITMAPCHIP_4_5_7, HITMAPCHIP_4_5_8, HITMAPCHIP_4_5_9,
    HITMAPCHIP_5_0_0, HITMAPCHIP_5_0_1, HITMAPCHIP_5_0_2, HITMAPCHIP_5_0_3, HITMAPCHIP_5_0_4,
    HITMAPCHIP_5_0_5, HITMAPCHIP_5_0_6, HITMAPCHIP_5_0_7, HITMAPCHIP_5_0_8, HITMAPCHIP_5_0_9,
    HITMAPCHIP_5_1_0, HITMAPCHIP_5_1_1, HITMAPCHIP_5_1_2, HITMAPCHIP_5_1_3, HITMAPCHIP_5_1_4,
    HITMAPCHIP_5_1_5, HITMAPCHIP_5_1_6, HITMAPCHIP_5_1_7, HITMAPCHIP_5_1_8, HITMAPCHIP_5_1_9,
    HITMAPCHIP_5_2_0, HITMAPCHIP_5_2_1, HITMAPCHIP_5_2_2, HITMAPCHIP_5_2_3, HITMAPCHIP_5_2_4,
    HITMAPCHIP_5_2_5, HITMAPCHIP_5_2_6, HITMAPCHIP_5_2_7, HITMAPCHIP_5_2_8, HITMAPCHIP_5_2_9,
    HITMAPCHIP_5_3_0, HITMAPCHIP_5_3_1, HITMAPCHIP_5_3_2, HITMAPCHIP_5_3_3, HITMAPCHIP_5_3_4,
    HITMAPCHIP_5_3_5, HITMAPCHIP_5_3_6, HITMAPCHIP_5_3_7, HITMAPCHIP_5_3_8, HITMAPCHIP_5_3_9,
    HITMAPCHIP_5_4_0, HITMAPCHIP_5_4_1, HITMAPCHIP_5_4_2, HITMAPCHIP_5_4_3, HITMAPCHIP_5_4_4,
    HITMAPCHIP_5_4_5, HITMAPCHIP_5_4_6, HITMAPCHIP_5_4_7, HITMAPCHIP_5_4_8, HITMAPCHIP_5_4_9,
    HITMAPCHIP_5_5_0, HITMAPCHIP_5_5_1, HITMAPCHIP_5_5_2, HITMAPCHIP_5_5_3, HITMAPCHIP_5_5_4,
    HITMAPCHIP_5_5_5, HITMAPCHIP_5_5_6, HITMAPCHIP_5_5_7, HITMAPCHIP_5_5_8, HITMAPCHIP_5_5_9,
    HITMAPCHIP_6_0_0, HITMAPCHIP_6_0_1, HITMAPCHIP_6_0_2, HITMAPCHIP_6_0_3, HITMAPCHIP_6_0_4,
    HITMAPCHIP_6_0_5, HITMAPCHIP_6_0_6, HITMAPCHIP_6_0_7, HITMAPCHIP_6_0_8, HITMAPCHIP_6_0_9,
    HITMAPCHIP_6_1_0, HITMAPCHIP_6_1_1, HITMAPCHIP_6_1_2, HITMAPCHIP_6_1_3, HITMAPCHIP_6_1_4,
    HITMAPCHIP_6_1_5, HITMAPCHIP_6_1_6, HITMAPCHIP_6_1_7, HITMAPCHIP_6_1_8, HITMAPCHIP_6_1_9,
    HITMAPCHIP_6_2_0, HITMAPCHIP_6_2_1, HITMAPCHIP_6_2_2, HITMAPCHIP_6_2_3, HITMAPCHIP_6_2_4,
    HITMAPCHIP_6_2_5, HITMAPCHIP_6_2_6, HITMAPCHIP_6_2_7, HITMAPCHIP_6_2_8, HITMAPCHIP_6_2_9,
    HITMAPCHIP_6_3_0, HITMAPCHIP_6_3_1, HITMAPCHIP_6_3_2, HITMAPCHIP_6_3_3, HITMAPCHIP_6_3_4,
    HITMAPCHIP_6_3_5, HITMAPCHIP_6_3_6, HITMAPCHIP_6_3_7, HITMAPCHIP_6_3_8, HITMAPCHIP_6_3_9,
    HITMAPCHIP_6_4_0, HITMAPCHIP_6_4_1, HITMAPCHIP_6_4_2, HITMAPCHIP_6_4_3, HITMAPCHIP_6_4_4,
    HITMAPCHIP_6_4_5, HITMAPCHIP_6_4_6, HITMAPCHIP_6_4_7, HITMAPCHIP_6_4_8, HITMAPCHIP_6_4_9,
    HITMAPCHIP_6_5_0, HITMAPCHIP_6_5_1, HITMAPCHIP_6_5_2, HITMAPCHIP_6_5_3, HITMAPCHIP_6_5_4,
    HITMAPCHIP_6_5_5, HITMAPCHIP_6_5_6, HITMAPCHIP_6_5_7, HITMAPCHIP_6_5_8, HITMAPCHIP_6_5_9,
    HITMAPCHIP_7_0_0, HITMAPCHIP_7_0_1, HITMAPCHIP_7_0_2, HITMAPCHIP_7_0_3, HITMAPCHIP_7_0_4,
    HITMAPCHIP_7_0_5, HITMAPCHIP_7_0_6, HITMAPCHIP_7_0_7, HITMAPCHIP_7_0_8, HITMAPCHIP_7_0_9,
    HITMAPCHIP_7_1_0, HITMAPCHIP_7_1_1, HITMAPCHIP_7_1_2, HITMAPCHIP_7_1_3, HITMAPCHIP_7_1_4,
    HITMAPCHIP_7_1_5, HITMAPCHIP_7_1_6, HITMAPCHIP_7_1_7, HITMAPCHIP_7_1_8, HITMAPCHIP_7_1_9,
    HITMAPCHIP_7_2_0, HITMAPCHIP_7_2_1, HITMAPCHIP_7_2_2, HITMAPCHIP_7_2_3, HITMAPCHIP_7_2_4,
    HITMAPCHIP_7_2_5, HITMAPCHIP_7_2_6, HITMAPCHIP_7_2_7, HITMAPCHIP_7_2_8, HITMAPCHIP_7_2_9,
    HITMAPCHIP_7_3_0, HITMAPCHIP_7_3_1, HITMAPCHIP_7_3_2, HITMAPCHIP_7_3_3, HITMAPCHIP_7_3_4,
    HITMAPCHIP_7_3_5, HITMAPCHIP_7_3_6, HITMAPCHIP_7_3_7, HITMAPCHIP_7_3_8, HITMAPCHIP_7_3_9,
    HITMAPCHIP_7_4_0, HITMAPCHIP_7_4_1, HITMAPCHIP_7_4_2, HITMAPCHIP_7_4_3, HITMAPCHIP_7_4_4,
    HITMAPCHIP_7_4_5, HITMAPCHIP_7_4_6, HITMAPCHIP_7_4_7, HITMAPCHIP_7_4_8, HITMAPCHIP_7_4_9,
    HITMAPCHIP_7_5_0, HITMAPCHIP_7_5_1, HITMAPCHIP_7_5_2, HITMAPCHIP_7_5_3, HITMAPCHIP_7_5_4,
    HITMAPCHIP_7_5_5, HITMAPCHIP_7_5_6, HITMAPCHIP_7_5_7, HITMAPCHIP_7_5_8, HITMAPCHIP_7_5_9,
    HITMAPCHIP_8_0_0, HITMAPCHIP_8_0_1, HITMAPCHIP_8_0_2, HITMAPCHIP_8_0_3, HITMAPCHIP_8_0_4,
    HITMAPCHIP_8_0_5, HITMAPCHIP_8_0_6, HITMAPCHIP_8_0_7, HITMAPCHIP_8_0_8, HITMAPCHIP_8_0_9,
    HITMAPCHIP_8_1_0, HITMAPCHIP_8_1_1, HITMAPCHIP_8_1_2, HITMAPCHIP_8_1_3, HITMAPCHIP_8_1_4,
    HITMAPCHIP_8_1_5, HITMAPCHIP_8_1_6, HITMAPCHIP_8_1_7, HITMAPCHIP_8_1_8, HITMAPCHIP_8_1_9,
    HITMAPCHIP_8_2_0, HITMAPCHIP_8_2_1, HITMAPCHIP_8_2_2, HITMAPCHIP_8_2_3, HITMAPCHIP_8_2_4,
    HITMAPCHIP_8_2_5, HITMAPCHIP_8_2_6, HITMAPCHIP_8_2_7, HITMAPCHIP_8_2_8, HITMAPCHIP_8_2_9,
    HITMAPCHIP_8_3_0, HITMAPCHIP_8_3_1, HITMAPCHIP_8_3_2, HITMAPCHIP_8_3_3, HITMAPCHIP_8_3_4,
    HITMAPCHIP_8_3_5, HITMAPCHIP_8_3_6, HITMAPCHIP_8_3_7, HITMAPCHIP_8_3_8, HITMAPCHIP_8_3_9,
    HITMAPCHIP_8_4_0, HITMAPCHIP_8_4_1, HITMAPCHIP_8_4_2, HITMAPCHIP_8_4_3, HITMAPCHIP_8_4_4,
    HITMAPCHIP_8_4_5, HITMAPCHIP_8_4_6, HITMAPCHIP_8_4_7, HITMAPCHIP_8_4_8, HITMAPCHIP_8_4_9,
    HITMAPCHIP_8_5_0, HITMAPCHIP_8_5_1, HITMAPCHIP_8_5_2, HITMAPCHIP_8_5_3, HITMAPCHIP_8_5_4,
    HITMAPCHIP_8_5_5, HITMAPCHIP_8_5_6, HITMAPCHIP_8_5_7, HITMAPCHIP_8_5_8, HITMAPCHIP_8_5_9,
    HITMAPCHIP_9_0_0, HITMAPCHIP_9_0_1, HITMAPCHIP_9_0_2, HITMAPCHIP_9_0_3, HITMAPCHIP_9_0_4,
    HITMAPCHIP_9_0_5, HITMAPCHIP_9_0_6, HITMAPCHIP_9_0_7, HITMAPCHIP_9_0_8, HITMAPCHIP_9_0_9,
    HITMAPCHIP_9_1_0, HITMAPCHIP_9_1_1, HITMAPCHIP_9_1_2, HITMAPCHIP_9_1_3, HITMAPCHIP_9_1_4,
    HITMAPCHIP_9_1_5, HITMAPCHIP_9_1_6, HITMAPCHIP_9_1_7, HITMAPCHIP_9_1_8, HITMAPCHIP_9_1_9,
    HITMAPCHIP_9_2_0, HITMAPCHIP_9_2_1, HITMAPCHIP_9_2_2, HITMAPCHIP_9_2_3, HITMAPCHIP_9_2_4,
    HITMAPCHIP_9_2_5, HITMAPCHIP_9_2_6, HITMAPCHIP_9_2_7, HITMAPCHIP_9_2_8, HITMAPCHIP_9_2_9,
    HITMAPCHIP_9_3_0, HITMAPCHIP_9_3_1, HITMAPCHIP_9_3_2, HITMAPCHIP_9_3_3, HITMAPCHIP_9_3_4,
    HITMAPCHIP_9_3_5, HITMAPCHIP_9_3_6, HITMAPCHIP_9_3_7, HITMAPCHIP_9_3_8, HITMAPCHIP_9_3_9,
    HITMAPCHIP_9_4_0, HITMAPCHIP_9_4_1, HITMAPCHIP_9_4_2, HITMAPCHIP_9_4_3, HITMAPCHIP_9_4_4,
    HITMAPCHIP_9_4_5, HITMAPCHIP_9_4_6, HITMAPCHIP_9_4_7, HITMAPCHIP_9_4_8, HITMAPCHIP_9_4_9,
    HITMAPCHIP_9_5_0, HITMAPCHIP_9_5_1, HITMAPCHIP_9_5_2, HITMAPCHIP_9_5_3, HITMAPCHIP_9_5_4,
    HITMAPCHIP_9_5_5, HITMAPCHIP_9_5_6, HITMAPCHIP_9_5_7, HITMAPCHIP_9_5_8, HITMAPCHIP_9_5_9,
    HITMAPCHIP_10_0_0, HITMAPCHIP_10_0_1, HITMAPCHIP_10_0_2, HITMAPCHIP_10_0_3, HITMAPCHIP_10_0_4,
    HITMAPCHIP_10_0_5, HITMAPCHIP_10_0_6, HITMAPCHIP_10_0_7, HITMAPCHIP_10_0_8, HITMAPCHIP_10_0_9,
    HITMAPCHIP_10_1_0, HITMAPCHIP_10_1_1, HITMAPCHIP_10_1_2, HITMAPCHIP_10_1_3, HITMAPCHIP_10_1_4,
    HITMAPCHIP_10_1_5, HITMAPCHIP_10_1_6, HITMAPCHIP_10_1_7, HITMAPCHIP_10_1_8, HITMAPCHIP_10_1_9,
    HITMAPCHIP_10_2_0, HITMAPCHIP_10_2_1, HITMAPCHIP_10_2_2, HITMAPCHIP_10_2_3, HITMAPCHIP_10_2_4,
    HITMAPCHIP_10_2_5, HITMAPCHIP_10_2_6, HITMAPCHIP_10_2_7, HITMAPCHIP_10_2_8, HITMAPCHIP_10_2_9,
    HITMAPCHIP_10_3_0, HITMAPCHIP_10_3_1, HITMAPCHIP_10_3_2, HITMAPCHIP_10_3_3, HITMAPCHIP_10_3_4,
    HITMAPCHIP_10_3_5, HITMAPCHIP_10_3_6, HITMAPCHIP_10_3_7, HITMAPCHIP_10_3_8, HITMAPCHIP_10_3_9,
    HITMAPCHIP_10_4_0, HITMAPCHIP_10_4_1, HITMAPCHIP_10_4_2, HITMAPCHIP_10_4_3, HITMAPCHIP_10_4_4,
    HITMAPCHIP_10_4_5, HITMAPCHIP_10_4_6, HITMAPCHIP_10_4_7, HITMAPCHIP_10_4_8, HITMAPCHIP_10_4_9,
    HITMAPCHIP_10_5_0, HITMAPCHIP_10_5_1, HITMAPCHIP_10_5_2, HITMAPCHIP_10_5_3, HITMAPCHIP_10_5_4,
    HITMAPCHIP_10_5_5, HITMAPCHIP_10_5_6, HITMAPCHIP_10_5_7, HITMAPCHIP_10_5_8, HITMAPCHIP_10_5_9,
    HITMAPCHIP_11_0_0, HITMAPCHIP_11_0_1, HITMAPCHIP_11_0_2, HITMAPCHIP_11_0_3, HITMAPCHIP_11_0_4,
    HITMAPCHIP_11_0_5, HITMAPCHIP_11_0_6, HITMAPCHIP_11_0_7, HITMAPCHIP_11_0_8, HITMAPCHIP_11_0_9,
    HITMAPCHIP_11_1_0, HITMAPCHIP_11_1_1, HITMAPCHIP_11_1_2, HITMAPCHIP_11_1_3, HITMAPCHIP_11_1_4,
    HITMAPCHIP_11_1_5, HITMAPCHIP_11_1_6, HITMAPCHIP_11_1_7, HITMAPCHIP_11_1_8, HITMAPCHIP_11_1_9,
    HITMAPCHIP_11_2_0, HITMAPCHIP_11_2_1, HITMAPCHIP_11_2_2, HITMAPCHIP_11_2_3, HITMAPCHIP_11_2_4,
    HITMAPCHIP_11_2_5, HITMAPCHIP_11_2_6, HITMAPCHIP_11_2_7, HITMAPCHIP_11_2_8, HITMAPCHIP_11_2_9,
    HITMAPCHIP_11_3_0, HITMAPCHIP_11_3_1, HITMAPCHIP_11_3_2, HITMAPCHIP_11_3_3, HITMAPCHIP_11_3_4,
    HITMAPCHIP_11_3_5, HITMAPCHIP_11_3_6, HITMAPCHIP_11_3_7, HITMAPCHIP_11_3_8, HITMAPCHIP_11_3_9,
    HITMAPCHIP_11_4_0, HITMAPCHIP_11_4_1, HITMAPCHIP_11_4_2, HITMAPCHIP_11_4_3, HITMAPCHIP_11_4_4,
    HITMAPCHIP_11_4_5, HITMAPCHIP_11_4_6, HITMAPCHIP_11_4_7, HITMAPCHIP_11_4_8, HITMAPCHIP_11_4_9,
    HITMAPCHIP_11_5_0, HITMAPCHIP_11_5_1, HITMAPCHIP_11_5_2, HITMAPCHIP_11_5_3, HITMAPCHIP_11_5_4,
    HITMAPCHIP_11_5_5, HITMAPCHIP_11_5_6, HITMAPCHIP_11_5_7, HITMAPCHIP_11_5_8, HITMAPCHIP_11_5_9,
    HITMAPCHIP_12_0_0, HITMAPCHIP_12_0_1, HITMAPCHIP_12_0_2, HITMAPCHIP_12_0_3, HITMAPCHIP_12_0_4,
    HITMAPCHIP_12_0_5, HITMAPCHIP_12_0_6, HITMAPCHIP_12_0_7, HITMAPCHIP_12_0_8, HITMAPCHIP_12_0_9,
    HITMAPCHIP_12_1_0, HITMAPCHIP_12_1_1, HITMAPCHIP_12_1_2, HITMAPCHIP_12_1_3, HITMAPCHIP_12_1_4,
    HITMAPCHIP_12_1_5, HITMAPCHIP_12_1_6, HITMAPCHIP_12_1_7, HITMAPCHIP_12_1_8, HITMAPCHIP_12_1_9,
    HITMAPCHIP_12_2_0, HITMAPCHIP_12_2_1, HITMAPCHIP_12_2_2, HITMAPCHIP_12_2_3, HITMAPCHIP_12_2_4,
    HITMAPCHIP_12_2_5, HITMAPCHIP_12_2_6, HITMAPCHIP_12_2_7, HITMAPCHIP_12_2_8, HITMAPCHIP_12_2_9,
    HITMAPCHIP_12_3_0, HITMAPCHIP_12_3_1, HITMAPCHIP_12_3_2, HITMAPCHIP_12_3_3, HITMAPCHIP_12_3_4,
    HITMAPCHIP_12_3_5, HITMAPCHIP_12_3_6, HITMAPCHIP_12_3_7, HITMAPCHIP_12_3_8, HITMAPCHIP_12_3_9,
    HITMAPCHIP_12_4_0, HITMAPCHIP_12_4_1, HITMAPCHIP_12_4_2, HITMAPCHIP_12_4_3, HITMAPCHIP_12_4_4,
    HITMAPCHIP_12_4_5, HITMAPCHIP_12_4_6, HITMAPCHIP_12_4_7, HITMAPCHIP_12_4_8, HITMAPCHIP_12_4_9,
    HITMAPCHIP_12_5_0, HITMAPCHIP_12_5_1, HITMAPCHIP_12_5_2, HITMAPCHIP_12_5_3, HITMAPCHIP_12_5_4,
    HITMAPCHIP_12_5_5, HITMAPCHIP_12_5_6, HITMAPCHIP_12_5_7, HITMAPCHIP_12_5_8, HITMAPCHIP_12_5_9,
    HITMAPCHIP_13_0_0, HITMAPCHIP_13_0_1, HITMAPCHIP_13_0_2, HITMAPCHIP_13_0_3, HITMAPCHIP_13_0_4,
    HITMAPCHIP_13_0_5, HITMAPCHIP_13_0_6, HITMAPCHIP_13_0_7, HITMAPCHIP_13_0_8, HITMAPCHIP_13_0_9,
    HITMAPCHIP_13_1_0, HITMAPCHIP_13_1_1, HITMAPCHIP_13_1_2, HITMAPCHIP_13_1_3, HITMAPCHIP_13_1_4,
    HITMAPCHIP_13_1_5, HITMAPCHIP_13_1_6, HITMAPCHIP_13_1_7, HITMAPCHIP_13_1_8, HITMAPCHIP_13_1_9,
    HITMAPCHIP_13_2_0, HITMAPCHIP_13_2_1, HITMAPCHIP_13_2_2, HITMAPCHIP_13_2_3, HITMAPCHIP_13_2_4,
    HITMAPCHIP_13_2_5, HITMAPCHIP_13_2_6, HITMAPCHIP_13_2_7, HITMAPCHIP_13_2_8, HITMAPCHIP_13_2_9,
    HITMAPCHIP_13_3_0, HITMAPCHIP_13_3_1, HITMAPCHIP_13_3_2, HITMAPCHIP_13_3_3, HITMAPCHIP_13_3_4,
    HITMAPCHIP_13_3_5, HITMAPCHIP_13_3_6, HITMAPCHIP_13_3_7, HITMAPCHIP_13_3_8, HITMAPCHIP_13_3_9,
    HITMAPCHIP_13_4_0, HITMAPCHIP_13_4_1, HITMAPCHIP_13_4_2, HITMAPCHIP_13_4_3, HITMAPCHIP_13_4_4,
    HITMAPCHIP_13_4_5, HITMAPCHIP_13_4_6, HITMAPCHIP_13_4_7, HITMAPCHIP_13_4_8, HITMAPCHIP_13_4_9,
    HITMAPCHIP_13_5_0, HITMAPCHIP_13_5_1, HITMAPCHIP_13_5_2, HITMAPCHIP_13_5_3, HITMAPCHIP_13_5_4,
    HITMAPCHIP_13_5_5, HITMAPCHIP_13_5_6, HITMAPCHIP_13_5_7, HITMAPCHIP_13_5_8, HITMAPCHIP_13_5_9,
    HITMAPCHIP_14_0_0, HITMAPCHIP_14_0_1, HITMAPCHIP_14_0_2, HITMAPCHIP_14_0_3, HITMAPCHIP_14_0_4,
    HITMAPCHIP_14_0_5, HITMAPCHIP_14_0_6, HITMAPCHIP_14_0_7, HITMAPCHIP_14_0_8, HITMAPCHIP_14_0_9,
    HITMAPCHIP_14_1_0, HITMAPCHIP_14_1_1, HITMAPCHIP_14_1_2, HITMAPCHIP_14_1_3, HITMAPCHIP_14_1_4,
    HITMAPCHIP_14_1_5, HITMAPCHIP_14_1_6, HITMAPCHIP_14_1_7, HITMAPCHIP_14_1_8, HITMAPCHIP_14_1_9,
    HITMAPCHIP_14_2_0, HITMAPCHIP_14_2_1, HITMAPCHIP_14_2_2, HITMAPCHIP_14_2_3, HITMAPCHIP_14_2_4,
    HITMAPCHIP_14_2_5, HITMAPCHIP_14_2_6, HITMAPCHIP_14_2_7, HITMAPCHIP_14_2_8, HITMAPCHIP_14_2_9,
    HITMAPCHIP_14_3_0, HITMAPCHIP_14_3_1, HITMAPCHIP_14_3_2, HITMAPCHIP_14_3_3, HITMAPCHIP_14_3_4,
    HITMAPCHIP_14_3_5, HITMAPCHIP_14_3_6, HITMAPCHIP_14_3_7, HITMAPCHIP_14_3_8, HITMAPCHIP_14_3_9,
    HITMAPCHIP_14_4_0, HITMAPCHIP_14_4_1, HITMAPCHIP_14_4_2, HITMAPCHIP_14_4_3, HITMAPCHIP_14_4_4,
    HITMAPCHIP_14_4_5, HITMAPCHIP_14_4_6, HITMAPCHIP_14_4_7, HITMAPCHIP_14_4_8, HITMAPCHIP_14_4_9,
    HITMAPCHIP_14_5_0, HITMAPCHIP_14_5_1, HITMAPCHIP_14_5_2, HITMAPCHIP_14_5_3, HITMAPCHIP_14_5_4,
    HITMAPCHIP_14_5_5, HITMAPCHIP_14_5_6, HITMAPCHIP_14_5_7, HITMAPCHIP_14_5_8, HITMAPCHIP_14_5_9,
    HITMAPCHIP_15_0_0, HITMAPCHIP_15_0_1, HITMAPCHIP_15_0_2, HITMAPCHIP_15_0_3, HITMAPCHIP_15_0_4,
    HITMAPCHIP_15_0_5, HITMAPCHIP_15_0_6, HITMAPCHIP_15_0_7, HITMAPCHIP_15_0_8, HITMAPCHIP_15_0_9,
    HITMAPCHIP_15_1_0, HITMAPCHIP_15_1_1, HITMAPCHIP_15_1_2, HITMAPCHIP_15_1_3, HITMAPCHIP_15_1_4,
    HITMAPCHIP_15_1_5, HITMAPCHIP_15_1_6, HITMAPCHIP_15_1_7, HITMAPCHIP_15_1_8, HITMAPCHIP_15_1_9,
    HITMAPCHIP_15_2_0, HITMAPCHIP_15_2_1, HITMAPCHIP_15_2_2, HITMAPCHIP_15_2_3, HITMAPCHIP_15_2_4,
    HITMAPCHIP_15_2_5, HITMAPCHIP_15_2_6, HITMAPCHIP_15_2_7, HITMAPCHIP_15_2_8, HITMAPCHIP_15_2_9,
    HITMAPCHIP_15_3_0, HITMAPCHIP_15_3_1, HITMAPCHIP_15_3_2, HITMAPCHIP_15_3_3, HITMAPCHIP_15_3_4,
    HITMAPCHIP_15_3_5, HITMAPCHIP_15_3_6, HITMAPCHIP_15_3_7, HITMAPCHIP_15_3_8, HITMAPCHIP_15_3_9,
    HITMAPCHIP_15_4_0, HITMAPCHIP_15_4_1, HITMAPCHIP_15_4_2, HITMAPCHIP_15_4_3, HITMAPCHIP_15_4_4,
    HITMAPCHIP_15_4_5, HITMAPCHIP_15_4_6, HITMAPCHIP_15_4_7, HITMAPCHIP_15_4_8, HITMAPCHIP_15_4_9,
    HITMAPCHIP_15_5_0, HITMAPCHIP_15_5_1, HITMAPCHIP_15_5_2, HITMAPCHIP_15_5_3, HITMAPCHIP_15_5_4,
    HITMAPCHIP_15_5_5, HITMAPCHIP_15_5_6, HITMAPCHIP_15_5_7, HITMAPCHIP_15_5_8, HITMAPCHIP_15_5_9,
    HITMAPCHIP_16_0_0, HITMAPCHIP_16_0_1, HITMAPCHIP_16_0_2, HITMAPCHIP_16_0_3, HITMAPCHIP_16_0_4,
    HITMAPCHIP_16_0_5, HITMAPCHIP_16_0_6, HITMAPCHIP_16_0_7, HITMAPCHIP_16_0_8, HITMAPCHIP_16_0_9,
    HITMAPCHIP_16_1_0, HITMAPCHIP_16_1_1, HITMAPCHIP_16_1_2, HITMAPCHIP_16_1_3, HITMAPCHIP_16_1_4,
    HITMAPCHIP_16_1_5, HITMAPCHIP_16_1_6, HITMAPCHIP_16_1_7, HITMAPCHIP_16_1_8, HITMAPCHIP_16_1_9,
    HITMAPCHIP_16_2_0, HITMAPCHIP_16_2_1, HITMAPCHIP_16_2_2, HITMAPCHIP_16_2_3, HITMAPCHIP_16_2_4,
    HITMAPCHIP_16_2_5, HITMAPCHIP_16_2_6, HITMAPCHIP_16_2_7, HITMAPCHIP_16_2_8, HITMAPCHIP_16_2_9,
    HITMAPCHIP_16_3_0, HITMAPCHIP_16_3_1, HITMAPCHIP_16_3_2, HITMAPCHIP_16_3_3, HITMAPCHIP_16_3_4,
    HITMAPCHIP_16_3_5, HITMAPCHIP_16_3_6, HITMAPCHIP_16_3_7, HITMAPCHIP_16_3_8, HITMAPCHIP_16_3_9,
    HITMAPCHIP_16_4_0, HITMAPCHIP_16_4_1, HITMAPCHIP_16_4_2, HITMAPCHIP_16_4_3, HITMAPCHIP_16_4_4,
    HITMAPCHIP_16_4_5, HITMAPCHIP_16_4_6, HITMAPCHIP_16_4_7, HITMAPCHIP_16_4_8, HITMAPCHIP_16_4_9,
    HITMAPCHIP_16_5_0, HITMAPCHIP_16_5_1, HITMAPCHIP_16_5_2, HITMAPCHIP_16_5_3, HITMAPCHIP_16_5_4,
    HITMAPCHIP_16_5_5, HITMAPCHIP_16_5_6, HITMAPCHIP_16_5_7, HITMAPCHIP_16_5_8, HITMAPCHIP_16_5_9,
    HITMAPCHIP_17_0_0, HITMAPCHIP_17_0_1, HITMAPCHIP_17_0_2, HITMAPCHIP_17_0_3, HITMAPCHIP_17_0_4,
    HITMAPCHIP_17_0_5, HITMAPCHIP_17_0_6, HITMAPCHIP_17_0_7, HITMAPCHIP_17_0_8, HITMAPCHIP_17_0_9,
    HITMAPCHIP_17_1_0, HITMAPCHIP_17_1_1, HITMAPCHIP_17_1_2, HITMAPCHIP_17_1_3, HITMAPCHIP_17_1_4,
    HITMAPCHIP_17_1_5, HITMAPCHIP_17_1_6, HITMAPCHIP_17_1_7, HITMAPCHIP_17_1_8, HITMAPCHIP_17_1_9,
    HITMAPCHIP_17_2_0, HITMAPCHIP_17_2_1, HITMAPCHIP_17_2_2, HITMAPCHIP_17_2_3, HITMAPCHIP_17_2_4,
    HITMAPCHIP_17_2_5, HITMAPCHIP_17_2_6, HITMAPCHIP_17_2_7, HITMAPCHIP_17_2_8, HITMAPCHIP_17_2_9,
    HITMAPCHIP_17_3_0, HITMAPCHIP_17_3_1, HITMAPCHIP_17_3_2, HITMAPCHIP_17_3_3, HITMAPCHIP_17_3_4,
    HITMAPCHIP_17_3_5, HITMAPCHIP_17_3_6, HITMAPCHIP_17_3_7, HITMAPCHIP_17_3_8, HITMAPCHIP_17_3_9,
    HITMAPCHIP_17_4_0, HITMAPCHIP_17_4_1, HITMAPCHIP_17_4_2, HITMAPCHIP_17_4_3, HITMAPCHIP_17_4_4,
    HITMAPCHIP_17_4_5, HITMAPCHIP_17_4_6, HITMAPCHIP_17_4_7, HITMAPCHIP_17_4_8, HITMAPCHIP_17_4_9,
    HITMAPCHIP_17_5_0, HITMAPCHIP_17_5_1, HITMAPCHIP_17_5_2, HITMAPCHIP_17_5_3, HITMAPCHIP_17_5_4,
    HITMAPCHIP_17_5_5, HITMAPCHIP_17_5_6, HITMAPCHIP_17_5_7, HITMAPCHIP_17_5_8, HITMAPCHIP_17_5_9,
    HITMAPCHIP_18_0_0, HITMAPCHIP_18_0_1, HITMAPCHIP_18_0_2, HITMAPCHIP_18_0_3, HITMAPCHIP_18_0_4,
    HITMAPCHIP_18_0_5, HITMAPCHIP_18_0_6, HITMAPCHIP_18_0_7, HITMAPCHIP_18_0_8, HITMAPCHIP_18_0_9,
    HITMAPCHIP_18_1_0, HITMAPCHIP_18_1_1, HITMAPCHIP_18_1_2, HITMAPCHIP_18_1_3, HITMAPCHIP_18_1_4,
    HITMAPCHIP_18_1_5, HITMAPCHIP_18_1_6, HITMAPCHIP_18_1_7, HITMAPCHIP_18_1_8, HITMAPCHIP_18_1_9,
    HITMAPCHIP_18_2_0, HITMAPCHIP_18_2_1, HITMAPCHIP_18_2_2, HITMAPCHIP_18_2_3, HITMAPCHIP_18_2_4,
    HITMAPCHIP_18_2_5, HITMAPCHIP_18_2_6, HITMAPCHIP_18_2_7, HITMAPCHIP_18_2_8, HITMAPCHIP_18_2_9,
    HITMAPCHIP_18_3_0, HITMAPCHIP_18_3_1, HITMAPCHIP_18_3_2, HITMAPCHIP_18_3_3, HITMAPCHIP_18_3_4,
    HITMAPCHIP_18_3_5, HITMAPCHIP_18_3_6, HITMAPCHIP_18_3_7, HITMAPCHIP_18_3_8, HITMAPCHIP_18_3_9,
    HITMAPCHIP_18_4_0, HITMAPCHIP_18_4_1, HITMAPCHIP_18_4_2, HITMAPCHIP_18_4_3, HITMAPCHIP_18_4_4,
    HITMAPCHIP_18_4_5, HITMAPCHIP_18_4_6, HITMAPCHIP_18_4_7, HITMAPCHIP_18_4_8, HITMAPCHIP_18_4_9,
    HITMAPCHIP_18_5_0, HITMAPCHIP_18_5_1, HITMAPCHIP_18_5_2, HITMAPCHIP_18_5_3, HITMAPCHIP_18_5_4,
    HITMAPCHIP_18_5_5, HITMAPCHIP_18_5_6, HITMAPCHIP_18_5_7, HITMAPCHIP_18_5_8, HITMAPCHIP_18_5_9,
    HITMAPCHIP_19_0_0, HITMAPCHIP_19_0_1, HITMAPCHIP_19_0_2, HITMAPCHIP_19_0_3, HITMAPCHIP_19_0_4,
    HITMAPCHIP_19_0_5, HITMAPCHIP_19_0_6, HITMAPCHIP_19_0_7, HITMAPCHIP_19_0_8, HITMAPCHIP_19_0_9,
    HITMAPCHIP_19_1_0, HITMAPCHIP_19_1_1, HITMAPCHIP_19_1_2, HITMAPCHIP_19_1_3, HITMAPCHIP_19_1_4,
    HITMAPCHIP_19_1_5, HITMAPCHIP_19_1_6, HITMAPCHIP_19_1_7, HITMAPCHIP_19_1_8, HITMAPCHIP_19_1_9,
    HITMAPCHIP_19_2_0, HITMAPCHIP_19_2_1, HITMAPCHIP_19_2_2, HITMAPCHIP_19_2_3, HITMAPCHIP_19_2_4,
    HITMAPCHIP_19_2_5, HITMAPCHIP_19_2_6, HITMAPCHIP_19_2_7, HITMAPCHIP_19_2_8, HITMAPCHIP_19_2_9,
    HITMAPCHIP_19_3_0, HITMAPCHIP_19_3_1, HITMAPCHIP_19_3_2, HITMAPCHIP_19_3_3, HITMAPCHIP_19_3_4,
    HITMAPCHIP_19_3_5, HITMAPCHIP_19_3_6, HITMAPCHIP_19_3_7, HITMAPCHIP_19_3_8, HITMAPCHIP_19_3_9,
    HITMAPCHIP_19_4_0, HITMAPCHIP_19_4_1, HITMAPCHIP_19_4_2, HITMAPCHIP_19_4_3, HITMAPCHIP_19_4_4,
    HITMAPCHIP_19_4_5, HITMAPCHIP_19_4_6, HITMAPCHIP_19_4_7, HITMAPCHIP_19_4_8, HITMAPCHIP_19_4_9,
    HITMAPCHIP_19_5_0, HITMAPCHIP_19_5_1, HITMAPCHIP_19_5_2, HITMAPCHIP_19_5_3, HITMAPCHIP_19_5_4,
    HITMAPCHIP_19_5_5, HITMAPCHIP_19_5_6, HITMAPCHIP_19_5_7, HITMAPCHIP_19_5_8, HITMAPCHIP_19_5_9,

    DUMMY_DUMMY
  };

  ClassDef(TMSetupDisplays, 0);
};

#endif
